#!/usr/local/env python
# -*- coding: utf-8 -*-
import argparse
import shlex
import sys

import nose
import prospector.run as prospector


class RunTests:
    description = 'Run lint and tests'

    def __init__(self):
        parsed_args = self.parse_arguments()
        self.test_module = parsed_args['test_module']
        self.test_args = shlex.split(' '.join(parsed_args['test_args']))
        self.skip_lint = parsed_args['skip_lint']
        self.skip_tests = parsed_args['skip_tests']
        self.exit_on_fail = parsed_args['exit_on_fail']

    def add_arguments(self, parser):
        parser.add_argument('--skip-lint', action='store_true', help='Skip lint')
        parser.add_argument('--skip-tests', action='store_true', help='Skip tests')
        parser.add_argument('-e', '--exit-on-fail', action='store_true', help='Exit on step fail')
        parser.add_argument('test_module', nargs='*', default=['.'], help='Module to test')
        parser.add_argument('--test_args', action='store', default=[], nargs=argparse.REMAINDER,
                            help='Extra arguments to pass to tests')

    def parse_arguments(self):
        parser = argparse.ArgumentParser(description=self.description)
        self.add_arguments(parser)
        return {k: v for k, v in vars(parser.parse_args()).items()}

    def tests(self):
        argv = ['nosetests'] + self.test_module + self.test_args
        result = nose.run(argv=argv)
        return (result + 1) % 2  # Change 0 to 1 and 1 to 0

    def lint(self):
        argv = sys.argv
        sys.argv = []
        result = prospector.main()
        sys.argv = argv
        return result

    def _check_exit(self, result):
        print('Return code: {:d}'.format(result))
        if self.exit_on_fail and result:
            sys.exit(result)

    def run(self):
        if not self.skip_lint:
            self._check_exit(self.lint())

        if not self.skip_tests:
            self._check_exit(self.tests())


if __name__ == '__main__':
    sys.exit(RunTests().run())



# -*- coding: utf-8 -*-
from functools import partial
from unittest.case import TestCase
from unittest.mock import MagicMock, patch, call

from celery.app.task import Task
from nose.plugins.attrib import attr

from task_dispatcher.decorators import BaseDecorator


class BaseDecoratorTestCase(TestCase):
    def setUp(self):
        self.task_mock = MagicMock(spec=Task)
        self.task_mock.__doc__ = 'docstring'
        self.task_mock.__qualname__ = 'qualname'
        self.task_mock.__module__ = 'module'
        self.task_mock.name = 'task_name'

    @attr(priority='high', speed='fast')
    def test_full_initialize_without_args(self):
        with patch.object(BaseDecorator, '_decorate') as decorate_mock:
            BaseDecorator(self.task_mock)
            self.assertEqual(decorate_mock.call_count, 1)

    @attr(priority='high', speed='fast')
    def test_partial_initialize_with_args(self):
        with patch.object(BaseDecorator, '_decorate') as decorate_mock:
            # Partial initialization to pass description
            decorator = BaseDecorator(description='description')

            self.assertEqual(decorate_mock.call_count, 0)

            # Full initialization through __call__
            decorator(self.task_mock)

            self.assertEqual(decorate_mock.call_count, 1)

    @attr(priority='high', speed='fast')
    def test_get(self):
        with patch('task_dispatcher.decorators.app') as celery_app_mock, \
                patch('task_dispatcher.decorators.register') as register_mock:
            celery_app_mock.task().return_value = self.task_mock

            decorator = BaseDecorator(self.task_mock)

        self.assertTrue(isinstance(decorator.__get__(decorator), partial))

    @attr(priority='high', speed='fast')
    def test_getattr_decorator_attr(self):
        with patch('task_dispatcher.decorators.app') as celery_app_mock, \
                patch('task_dispatcher.decorators.register') as register_mock:
            celery_app_mock.task().return_value = self.task_mock

            decorator = BaseDecorator(description='description')(self.task_mock)

        description = decorator.description
        expected_description = 'description'

        self.assertEqual(expected_description, description)

    @attr(priority='high', speed='fast')
    def test_getattr_task_attr(self):
        with patch('task_dispatcher.decorators.app') as celery_app_mock, \
                patch('task_dispatcher.decorators.register') as register_mock:
            celery_app_mock.task().return_value = self.task_mock

            decorator = BaseDecorator(description='description')(self.task_mock)

        name = decorator.name
        expected_name = 'task_name'

        self.assertEqual(expected_name, name)

    @attr(priority='high', speed='fast')
    def test_decorate(self):
        with patch('task_dispatcher.decorators.app') as celery_app_mock, \
                patch('task_dispatcher.decorators.register') as register_mock:
            celery_app_mock.task().return_value = self.task_mock

            decorator = BaseDecorator(self.task_mock)

        self.assertEqual(self.task_mock, decorator.task)
        self.assertEqual(register_mock.register.call_count, 1)

    @attr(priority='high', speed='fast')
    def test_call(self):
        expected_call_args = [call('foo', bar='bar')]

        with patch('task_dispatcher.decorators.app') as celery_app_mock, \
                patch('task_dispatcher.decorators.register') as register_mock:
            # Partial initialization to pass description
            celery_app_mock.task().return_value = self.task_mock
            decorator = BaseDecorator(description='description')(self.task_mock)

            # Full initialization through __call__
            decorator('foo', bar='bar')

            self.assertEqual(self.task_mock, decorator.task)
            self.assertEqual(self.task_mock.call_count, 1)
            self.assertCountEqual(expected_call_args, self.task_mock.call_args_list)

    @attr(priority='high', speed='fast')
    def test_call_not_initialized(self):
        expected_call_args = [call('foo', bar='bar')]

        with patch('task_dispatcher.decorators.app') as celery_app_mock, \
                patch('task_dispatcher.decorators.register') as register_mock:
            # Partial initialization to pass description
            celery_app_mock.task().return_value = self.task_mock
            decorator = BaseDecorator(description='description')

            # Try to call before full initialization
            self.assertRaises(ValueError, decorator, 'foo', bar='bar')



# -*- coding: utf-8 -*-
import sys
from unittest.case import TestCase
from unittest.mock import patch

from nose.plugins.attrib import attr

from task_dispatcher.commands import TaskDispatcherCommand


class TaskDispatcherCommandTestCase(TestCase):
    @attr(priority='mid', speed='fast')
    def test_consumer(self):
        expected_kwarg = 'queues'
        expected_queues = ['consumer']

        sys.argv = ['command', 'consumer']
        command = TaskDispatcherCommand()

        with patch('task_dispatcher.commands.app') as celery_app_mock:
            command.run()

        self.assertEqual(celery_app_mock.Worker.call_count, 1)
        self.assertIn(expected_kwarg, celery_app_mock.Worker.call_args[1])
        self.assertCountEqual(expected_queues, celery_app_mock.Worker.call_args[1]['queues'])

    @attr(priority='mid', speed='fast')
    def test_producer(self):
        expected_kwarg = 'queues'
        expected_queues = ['producer']

        sys.argv = ['command', 'producer']
        command = TaskDispatcherCommand()

        with patch('task_dispatcher.commands.app') as celery_app_mock:
            command.run()

        self.assertEqual(celery_app_mock.Worker.call_count, 1)
        self.assertIn(expected_kwarg, celery_app_mock.Worker.call_args[1])
        self.assertCountEqual(expected_queues, celery_app_mock.Worker.call_args[1]['queues'])

    @attr(priority='mid', speed='fast')
    def test_scheduler(self):
        sys.argv = ['command', 'scheduler']
        command = TaskDispatcherCommand()

        with patch('task_dispatcher.commands.app') as celery_app_mock:
            command.run()

        self.assertEqual(celery_app_mock.Beat.call_count, 1)

    @attr(priority='mid', speed='fast')
    def test_list_yaml(self):
        sys.argv = ['command', 'list']
        command = TaskDispatcherCommand()

        with patch('task_dispatcher.commands.register') as register_mock:
            command.run()

        self.assertEqual(register_mock.to_yaml.call_count, 1)

    @attr(priority='mid', speed='fast')
    def test_list_json(self):
        sys.argv = ['command', 'list', '-f', 'json']
        command = TaskDispatcherCommand()

        with patch('task_dispatcher.commands.register') as register_mock:
            command.run()

        self.assertEqual(register_mock.to_json.call_count, 1)

    @attr(priority='low', speed='fast')
    def test_run_no_args(self):
        command = TaskDispatcherCommand(parse_args=False)

        self.assertRaises(ValueError, command.run)



# -*- coding: utf-8 -*-
"""Django application config module.
"""
try:
    from django.apps import AppConfig
    from django.utils.translation import ugettext_lazy as _
except ImportError:
    AppConfig = object

    def _(x): return x


class TaskDispatcher(AppConfig):
    name = 'task_dispatcher'
    verbose_name = _('Task Dispatcher')



# -*- coding: utf-8 -*-
from task_dispatcher.celery import app
from task_dispatcher.decorators import producer, consumer
from task_dispatcher.register import register

__version__ = '1.1.4'
__license__ = 'GPLv3'

__author__ = 'José Antonio Perdiguero López'
__email__ = 'perdy.hh@gmail.com'

__url__ = 'https://github.com/PeRDy/task-dispatcher'
__description__ = 'Library that provides a system to generate tasks producers and consumers with ease.'

__all__ = ['producer', 'consumer', 'register', 'app']

default_app_config = 'task_dispatcher.apps.TaskDispatcher'



# -*- coding: utf-8 -*-
from argparse import ArgumentParser
from typing import Dict, Any

from celery.bin.beat import beat as beat_bin
from celery.bin.worker import worker as worker_bin

from task_dispatcher.celery import app
from task_dispatcher.register import register


class TaskDispatcherCommand:
    """
    Command for running producer, consumer and scheduler processes along with some other utilities. This acts as a
    Command Line Interface for Task dispatcher.
    """

    LIST_JSON = 'json'
    LIST_YAML = 'yaml'
    LIST_CHOICES = (LIST_JSON, LIST_YAML)
    description = 'Task dispatcher command that provides a common entry point for running the different processes as ' \
                  'well as some utilities.'

    def __init__(self, parse_args=True):
        """
        Command for running producer, consumer and scheduler processes along with some other utilities.

        :param parse_args: If true, parse sys args.
        """
        self.args = self.parse_arguments() if parse_args else {}

    def parse_arguments(self) -> Dict[str, Any]:
        """
        Parse sys args and transform it into a dict.

        :return: Arguments parsed in dict form.
        """
        parser = ArgumentParser(description=self.description)
        self.add_arguments(parser)
        return {k: v for k, v in vars(parser.parse_args()).items()}

    def add_arguments(self, parser):
        """
        Add command arguments to a given parser.

        For consumer and producer subcommands the arguments are inherited directly from Celery worker command. In the
        case of scheduler subcommand, the arguments will also be inherited from Celery beat command.

        :param parser: Argument parser.
        """
        subparsers = parser.add_subparsers(title='Commands', help='Task dispatcher commands')

        wb = worker_bin(app=app)

        parser_consumer = subparsers.add_parser('consumer', help='Run a consumer')
        wb.add_arguments(parser_consumer)
        parser_consumer.set_defaults(func=self.consumer)

        parser_producer = subparsers.add_parser('producer', help='Run a producer')
        wb.add_arguments(parser_producer)
        parser_producer.set_defaults(func=self.producer)

        bb = beat_bin(app=app)

        parser_scheduler = subparsers.add_parser('scheduler', help='Run the tasks scheduler')
        bb.add_arguments(parser_scheduler)
        parser_scheduler.set_defaults(func=self.scheduler)

        parser_register = subparsers.add_parser('list', help='List task registered')
        parser_register.add_argument('-f', '--format', choices=self.LIST_CHOICES, default=self.LIST_YAML)
        parser_register.set_defaults(func=self.list)

    def consumer(self, *args, **kwargs) -> int:
        """
        Run a consumer process.
        """
        kwargs['queues'] = kwargs['queues'] or ['consumer']
        worker = app.Worker(**kwargs)
        worker.start()
        return worker.exitcode

    def producer(self, *args, **kwargs) -> int:
        """
        Run a producer process.
        """
        kwargs['queues'] = kwargs['queues'] or ['producer']
        worker = app.Worker(**kwargs)
        worker.start()
        return worker.exitcode

    def scheduler(self, *args, **kwargs):
        """
        Run a scheduler process.
        """
        beat = app.Beat(**kwargs)
        return beat.run()

    def list(self, *args, **kwargs):
        """
        Lists all producers and consumers registered.
        """
        if kwargs['format'] == self.LIST_YAML:
            print(register.to_yaml())
        else:
            print(register.to_json())

    def run(self, **kwargs):
        """
        Command entrypoint.
        """
        kwargs = kwargs or self.args

        if not kwargs:
            raise ValueError('Arguments must be passed or parsed previously')

        command = kwargs['func']

        return command(**kwargs)



# -*- coding: utf-8 -*-
import json
from collections import OrderedDict
from unittest import TestCase
from unittest.mock import patch, MagicMock

import yaml
from celery.app.task import Task
from nose.plugins.attrib import attr

from task_dispatcher.decorators import consumer, producer
from task_dispatcher.register import Register, TaskRegister


class RegisterTestCase(TestCase):
    def setUp(self):
        self.register = Register()

    def test_set_new_item(self):
        self.register['foo'] = 'bar'

        self.assertIn('foo', self.register)
        self.assertEqual(self.register['foo'], 'bar')

    def test_set_existing_item(self):
        self.register['foo'] = 'bar'

        with self.assertRaises(KeyError):
            self.register['foo'] = 'foobar'

    def test_str(self):
        expected_result = str({'foo': 'bar'})

        self.register['foo'] = 'bar'

        self.assertEqual(expected_result, str(self.register))

    def test_repr(self):
        expected_result = repr({'foo': 'bar'})

        self.register['foo'] = 'bar'

        self.assertEqual(expected_result, repr(self.register))


class TaskRegisterTestCase(TestCase):
    @classmethod
    def setUpClass(cls):
        producer_mock = MagicMock(spec=Task)
        producer_mock.__doc__ = 'docstring'
        producer_mock.__qualname__ = 'qualname'
        producer_mock.__module__ = 'module'
        producer_mock.name = 'producer_name'

        consumer_mock = MagicMock(spec=Task)
        consumer_mock.__doc__ = 'docstring'
        consumer_mock.__qualname__ = 'qualname'
        consumer_mock.__module__ = 'module'
        consumer_mock.name = 'consumer_name'

        with patch('task_dispatcher.decorators.app') as celery_app_mock:
            celery_app_mock.task().side_effect = [producer_mock, consumer_mock]
            cls.tasks = {
                'producer': producer(description='description')(producer_mock),
                'consumer': consumer(consumer_mock)
            }

    def setUp(self):
        self.register = TaskRegister()

    @attr(priority='high', speed='fast')
    def test_register_producer(self):
        self.register.register(self.tasks['producer'])

        self.assertIn(self.tasks['producer'], self.register.producers.values())

    @attr(priority='high', speed='fast')
    def test_register_consumer(self):
        self.register.register(self.tasks['consumer'])

        self.assertIn(self.tasks['consumer'], self.register.consumers.values())

    @attr(priority='high', speed='fast')
    def test_register_unknown(self):
        self.assertRaises(TypeError, self.register.register, False)

    @attr(priority='high', speed='fast')
    def test_consumers(self):
        self.register.register(self.tasks['consumer'])

        self.assertIn(self.tasks['consumer'], self.register.consumers.values())

    @attr(priority='high', speed='fast')
    def test_producers(self):
        self.register.register(self.tasks['producer'])

        self.assertIn(self.tasks['producer'], self.register.producers.values())

    @attr(priority='high', speed='fast')
    def test_to_dict(self):
        expected_result = {
            'consumers': {
                'consumer_name': {
                    'description': 'docstring',
                    'module': 'module',
                    'name': 'qualname',
                },
            },
            'producers': {
                'producer_name': {
                    'description': 'description',
                    'module': 'module',
                    'name': 'qualname',
                }
            }
        }

        self.register.register(self.tasks['consumer'])
        self.register.register(self.tasks['producer'])

        result = self.register.to_dict()

        self.assertDictEqual(expected_result, result)

    @attr(priority='high', speed='fast')
    def test_to_json(self):
        expected_result = json.dumps(OrderedDict(sorted({
            'consumers': {
                'consumer_name': {
                    'description': 'docstring',
                    'module': 'module',
                    'name': 'qualname',
                },
            },
            'producers': {
                'producer_name': {
                    'description': 'description',
                    'module': 'module',
                    'name': 'qualname',
                }
            }
        }.items())))

        self.register.register(self.tasks['consumer'])
        self.register.register(self.tasks['producer'])

        result = self.register.to_json()

        self.assertEqual(expected_result, result)

    @attr(priority='high', speed='fast')
    def test_to_yaml(self):
        expected_result = yaml.safe_dump({
            'consumers': {
                'consumer_name': {
                    'description': 'docstring',
                    'module': 'module',
                    'name': 'qualname',
                },
            },
            'producers': {
                'producer_name': {
                    'description': 'description',
                    'module': 'module',
                    'name': 'qualname',
                }
            }
        }, default_flow_style=False)

        self.register.register(self.tasks['consumer'])
        self.register.register(self.tasks['producer'])

        result = self.register.to_yaml()

        self.assertEqual(expected_result, result)



# -*- coding: utf-8 -*-
from functools import partial
from functools import update_wrapper
from typing import Callable

from task_dispatcher.celery import app
from task_dispatcher.register import register

__all__ = ['producer', 'consumer']


class BaseDecorator(object):
    """
    Decorator that creates a Celery task of given function or class method and register it.
    """

    def __init__(self, func: Callable=None, description: str=None, *args, **kwargs):
        """
        Decorator that creates a Celery task of given function or class method and register it. This decorator allows
        to be used as a common decorator without arguments:

        @BaseDecorator
        def foo(bar):
            pass

        But also is possible to provide some arguments, such as a task description:
        @BaseDecorator(description='Foo task that does nothing')
        def foo(bar):
            pass

        It's also possible to pass arguments to Celery task decorator, e.g:
        @BaseDecorator(queue='base_queue')
        def foo(bar):
            pass

        For last, is possible to decorate functions or class methods:
        class Foo:
            @BaseDecorator
            def bar(self):
                pass

        :param func: Function or class method to be decorated.
        :param description: Task description.
        :param args: Celery task args.
        :param kwargs: Celery task kwargs.
        """
        self.description = description
        self.args = args
        self.kwargs = kwargs

        if func is not None:
            # Full initialization decorator
            self._decorate(func, *args, **kwargs)
        else:
            # Partial initialization decorator
            self.task = None

    def _decorate(self, func: Callable, *args, **kwargs):
        self.task = app.task(*args, **kwargs)(func)
        update_wrapper(self, func)

        register.register(self)

    def __get__(self, instance, owner=None):
        """
        Make it works with functions and methods.
        """
        return partial(self, instance)

    def __getattr__(self, item):
        """
        Make this decorator a simple proxy for task instance.
        """
        # If looking for an unknown attr, delegates it to task __getattr__
        if self.task:
            return getattr(self.task, item)

    def __call__(self, *args, **kwargs):
        """
        Redirect calls to task instance. If decorator is not fully initialized, initialize it.
        """
        if self.task:
            # Decorator behavior
            return self.task(*args, **kwargs)
        else:
            # Decorator is not initialized and now is giving the function to be decorated
            if len(args) == 1 and len(kwargs) == 0 and callable(args[0]):
                self._decorate(func=args[0], *self.args, **self.kwargs)
                return self
            else:
                raise ValueError('Decorator is not initialized')


class producer(BaseDecorator):  # noqa
    """
    Decorator that creates a Celery task of given function or class method and register it. This tasks acts as a
    producer task.
    """
    pass


class consumer(BaseDecorator):  # noqa
    """
    Decorator that creates a Celery task of given function or class method and register it. This tasks acts as a
    consumer task.
    """
    pass



# -*- coding: utf-8 -*-
import json
from collections import OrderedDict

import yaml

__all__ = ['register', 'TaskRegister', 'Register']


class Register(dict):
    """
    Special case of dictionary that when keys cannot be replaced.
    """
    def __setitem__(self, key, value):
        if key in self:
            raise KeyError('{} is already registered'.format(key))

        dict.__setitem__(self, key, value)

    def __repr__(self):
        return dict.__repr__(self)

    def __str__(self):
        return dict.__str__(self)


class TaskRegister:
    """
    Register for producer and consumer tasks.
    """
    def __init__(self):
        """
        Register for producer and consumer tasks.
        """
        self._producers = Register()
        self._consumers = Register()

    def register(self, item):
        """
        Register a new task. The task will be discriminated and stored in their assigned register.

        :param item: Task to be registered.
        :raise TypeError: Item is not a registrable task.
        """
        from task_dispatcher.decorators import producer, consumer

        if isinstance(item, producer):
            self._producers[item.name] = item
        elif isinstance(item, consumer):
            self._consumers[item.name] = item
        else:
            raise TypeError(item)

    @property
    def producers(self):
        """
        Producers register.

        :return: Producers register.
        """
        return self._producers

    @property
    def consumers(self):
        """
        Consumers register.

        :return: Consumers register.
        """
        return self._consumers

    def to_dict(self) -> dict:
        """
        Transform the task register to a dictionary.

        :return: Task register transformed.
        """
        def get_description(task):
            return {
                'description': task.description or task.__doc__ or 'Description not found',
                'module': task.__module__,
                'name': task.__qualname__,
            }

        return OrderedDict([
            ('consumers', {k: get_description(v) for k, v in self._consumers.items()}),
            ('producers', {k: get_description(v) for k, v in self._producers.items()}),
        ])

    def to_json(self) -> str:
        """
        Transform the task register to a JSON string.

        :return: Task register transformed.
        """
        return json.dumps(self.to_dict())

    def to_yaml(self) -> str:
        """
        Transform the task register to a YAML string.

        :return: Task register transformed.
        """
        return yaml.dump(dict(self.to_dict()), default_flow_style=False)


register = TaskRegister()



# -*- coding: utf-8 -*-
import sys

from task_dispatcher.commands import TaskDispatcherCommand

__all__ = ['main']


def main():
    return TaskDispatcherCommand().run()


if __name__ == '__main__':
    sys.exit(main())



# -*- coding: utf-8 -*-

import os
import shutil
import sys

from pip.download import PipSession
from pip.req import parse_requirements as requirements
from setuptools import setup, Command

BASE_DIR = os.path.abspath(os.path.dirname(__file__))

if sys.version_info[0] == 2:
    from codecs import open


def parse_requirements(f):
    return [str(r.req) for r in requirements(f, session=PipSession())]


class Dist(Command):
    description = 'Generate static files and create dist package'
    user_options = [
        ('clean', 'c', 'clean dist directories before build (default: false)')
    ]
    boolean_options = ['clean']

    def initialize_options(self):
        self.clean = False

    def finalize_options(self):
        pass

    def run(self):
        if self.clean:
            shutil.rmtree('build', ignore_errors=True)
            shutil.rmtree('dist', ignore_errors=True)
            shutil.rmtree('health_check.egg-info', ignore_errors=True)

        self.run_command('sdist')
        self.run_command('bdist_wheel')

# Read requirements
_requirements_file = os.path.join(BASE_DIR, 'requirements.txt')
_tests_requirements_file = os.path.join(BASE_DIR, 'tests/requirements.txt')
_REQUIRES = parse_requirements(_requirements_file)
_TESTS_REQUIRES = parse_requirements(_tests_requirements_file)

# Read description
with open(os.path.join(BASE_DIR, 'README.rst'), encoding='utf-8') as f:
    _LONG_DESCRIPTION = f.read()

_CLASSIFIERS = (
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    'Natural Language :: English',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Software Development :: Object Brokering',
)
_KEYWORDS = ' '.join([
    'python',
    'stream',
    'data',
    'dispatcher',
    'flow',
    'database',
    'cache',
    'celery',
    'producer',
    'consumer',
    'scheduler',
])

setup(
    name='task-dispatcher',
    version='1.1.4',
    description='Library that provides a system to generate tasks producers and consumers with ease.',
    long_description=_LONG_DESCRIPTION,
    author='José Antonio Perdiguero López',
    author_email='perdy.hh@gmail.com',
    maintainer='José Antonio Perdiguero López',
    maintainer_email='perdy.hh@gmail.com',
    url='https://github.com/PeRDy/task-dispatcher',
    download_url='https://github.com/PeRDy/task-dispatcher',
    packages=[
        'task_dispatcher',
    ],
    include_package_data=True,
    install_requires=_REQUIRES,
    tests_require=_TESTS_REQUIRES,
    extras_require={
        'dev': [
            'setuptools',
            'pip',
            'wheel',
            'prospector',
            'twine',
            'bumpversion',
            'pre-commit',
            'nose'
            'tox',
        ]
    },
    license='GPLv3',
    zip_safe=False,
    keywords=_KEYWORDS,
    classifiers=_CLASSIFIERS,
    test_suite='nose.collector',
    cmdclass={
        'dist': Dist,
    },
    entry_points={
        'console_scripts': [
            'task_dispatcher = task_dispatcher.__main__:main',
        ],
    },
)
